//  StartVEFrame.java - jveframe application
//  Pietro Terna (October 2000 - November 2002)
//  Dipartimento di Scienze economiche e finanziarie G.Prato
//  Universit di Torino

//  Some building ideas and comments come from jHeatbugs (Java Heatbugs
//  application. Copyright  1999-2000 Swarm Development Group.)

import swarm.Globals;
import java.io.*;

/**
 * The StartVEFrame class contains main().
 * We follow here the typical Swarm structure with main() (in Start...
 * as a convention) generating the Observer and the Observer
 * generating the Model.<br>
 *
 * @author Pietro Terna and several of his students
 */
public class StartVEFrame
    {
	/** the observer in our application */
	public static VEFrameObserverSwarm vEFrameObserverSwarm;

	/** being verbose? */
	public static boolean verbose;

	/**
	 * The main() function is the top-level place where everything starts.
	 */
    public static void main (String[] args) {


	// Swarm initialization
        Globals.env.initSwarm ("jveframe", "v.0.9.7.33", 
			       "pietro.terna@unito.it",args);

	// creating the VEFrameObserverSwarm via lispAppArchiver, which
	// refers to the jveframe.scm file where we can modify parameters
	// in a stable way without recompile the interested class
        vEFrameObserverSwarm =
	   (VEFrameObserverSwarm) Globals.env.lispAppArchiver.getWithZone$key(
			      Globals.env.globalZone, "vEFrameObserverSwarm");
	// instead of using the direct constructor way
	// vEFrameObserverSwarm = new
	//                      VEFrameObserverSwarm (Globals.env.globalZone);

	// to save control panel position
	Globals.env.setWindowGeometryRecordName
	    (vEFrameObserverSwarm, "vEFrameObserverSwarm");

	// build objets into the obsever
	vEFrameObserverSwarm.buildObjects ();
	// verbose now is true/false, following the choice made
	// in the Observer

	// build actions into the observer
	vEFrameObserverSwarm.buildActions ();

	// activate
	vEFrameObserverSwarm.activateIn (null);

	// go() in vEFrameObserver is a method inherited from class
	// swarm.simtoolsgui.GUISwarmImpl
	// go() start the activity running, and also handle user requests
	// via the control panel.
	vEFrameObserverSwarm.go();
	
	// concluding the activy we close the order log file
	try{
	    if(vEFrameObserverSwarm.vEFrameModelSwarm.
	       concludedOrderLog != null)
		vEFrameObserverSwarm.vEFrameModelSwarm.
		    concludedOrderLog.close();
	    vEFrameObserverSwarm.vEFrameModelSwarm.
		concludedOrderLog=null;
	}
	catch (IOException e){
	    System.out.println(e);
	    System.exit(1);
	}

	// concluding the activity we drop the observer and then the model
	// (these operations are necessary to avoid errors in the closing
	// sequence, due to the final exit, necesary to close the histograms
	// which are not generated by Swarm tools
	// dropping the observer we use a redefined drop method, which
	// starts with the 'getControlPanel ().setStateStopped ();' instruction
	// necessary to avoid a Tcl error, appearing in W2000 and XP
	// we have so to press Quit twice
	vEFrameObserverSwarm.drop();
	VEFrameModelSwarm ourModel = vEFrameObserverSwarm.vEFrameModelSwarm;
	ourModel.drop();

	// and we quit also the execution of java graphs
	System.exit(0);
    }

}
