// OrderGenerator.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

import swarm.collections.ListImpl;

/**
 * The OrderGenerator is used at present to simulate the market
 * behavior.
 *
 * @author Pietro Terna */
public class OrderGenerator  extends SwarmObjectImpl{

    /** How many unit we have, used at present to define the max value
	used to generate the 'digits' composing our recipes */

    /** the max length of a recipe */
    public int maxStepNumber;

    /** the max length of a step in a recipe */
    public int maxStepLength;

    /** used to record the number of generated orders*/
    public int orderCount;

    /** a switch used in 'or' order generation */
    public int use0or1=0;

    /** the total number of layers;      
     *  a layer is a period of time or a set of qualitative conditions that 
     *  introduce differentiations into the orders; e.g two collections in 
     *  fashion production, with the same technical description (recipe) and 
     *  different qualitative results */
    public int totalLayerNumber;
    /** a layer number */
    public int layerNumber=0;

    /** a specific order*/
    public Order anOrder;
    /** the list containig the operating units*/
    public ListImpl unitList;
   /** the list containig the end units*/
    public ListImpl endUnitList;
    /** the list containig all the orders*/
    public ListImpl orderList;
    /** the model (to which we apply mark up of the order revenue */
    VEFrameModelSwarm vEFrameModelSwarm;

    /** the integer vector containing a specific production recipe */
    public int[] orderRecipe1; 

    /** the integer vector containing the unit of time spent for each step 
     *  in the recipe; il will determine the tranformation of the orderRecipe1 
     *  vector into the orderRecipe2 vector containing the repetitions
     *  of the steps, to reproduce the length of each step in time
     *
     *  NB this formalism is internal to internal (in external formalism
     *  we have 'step s/h/d time'
     *
     *  as an examples
     * 
     *  orderRecipe1      contains  1 12  7  3
     *  stepLengthInOrder contains  1  3  2  2
     *
     *  orderRecipe2      contains  1 12 12 12  7  7  3  3 
     */
    public int[] stepLengthInOrder;

    /** the integer vector containing a specific production recipe,
     *  developed following the length of each step
     */
    public int[] orderRecipe2;

    /** a temporary vector to redefine the dimension of orderRecipe2
     *  if necessary (using  "p" or "c" steps) */ 
    public int[] orderRecipeTmp;

    /** the dictionary to be used to choose the steps to be included in 
        a recipe */
    public int[] dictionary;
    /** length of the dictionary */
    public int dictionaryLength;
 
    /** the tool used in assigning orders */
    AssigningTool assigningTool;

    // units
    Unit aUnit;
    EndUnit anEndUnit;

    /**
     * the constructor for OrderGenerator
     */
    public OrderGenerator (Zone aZone, int msn, int msl, ListImpl ul, 
			   ListImpl eul, ListImpl ol, int tln,
			   VEFrameModelSwarm mo, AssigningTool at) 
    {

	// Call the constructor for the OrderGenerator parent class.
	super(aZone); int dim;

	orderCount=0;
	maxStepNumber=msn;
	if(maxStepNumber==0){
	    System.out.println("maxStepNumber cannot be 0 in this run.");
	    MyExit.exit(1);}
	maxStepLength=msl;
	if(maxStepLength==0){
	    System.out.println("maxStepLength cannot be 0 in this run.");
	    MyExit.exit(1);}
	unitList=ul;
	endUnitList=eul;
        orderList=ol;

	totalLayerNumber=tln;
	orderRecipe1 = new int[maxStepNumber];
	stepLengthInOrder = new int[maxStepNumber];
	dim=maxStepNumber*maxStepLength; if(dim<10)dim=10;
	orderRecipe2 = new int[dim];

        vEFrameModelSwarm = mo;
	assigningTool=at;
    }

    /** building the dictionary containing the steps to be included in the
	recipes */
    public void setDictionary()
    {
	int i, ii, iii;
	// how many words in the dictionary?

	// starting point: the no. of endUnits 
	dictionaryLength=endUnitList.getCount(); 

	for (i=0;i<unitList.getCount();i++)
	    {
		aUnit=(Unit) unitList.atOffset(i);
		dictionaryLength += aUnit.getNOfPhasesToDealWith();
	    }   
	if(StartVEFrame.verbose)
	    System.out.println(
	 "Length of the dictionary (# of codes identifying steps in recipes): "
	 + dictionaryLength);

	//creating the dictionary
	dictionary = new int[dictionaryLength];

	// loading terms into the dictionary (from endUnits and then units)
	i=0;
	for (ii=0;ii<endUnitList.getCount();ii++)
	    {
		anEndUnit=(EndUnit) endUnitList.atOffset(ii);
		dictionary[i++]=anEndUnit.getUnitNumber();
	    }
	for (ii=0;ii<unitList.getCount();ii++)
	    {
		aUnit=(Unit) unitList.atOffset(ii);
		for (iii=0;iii<aUnit.getNOfPhasesToDealWith();iii++)
		    dictionary[i++]=aUnit.getProductionPhase(iii);
	    }

	if(StartVEFrame.verbose)
	    for (i=0;i<dictionaryLength;i++)
		System.out.println(i + "  " + dictionary[i]);
    }


    /** creating an order form, choosing its recipe length and content */
    public void createRandomOrderWithNSteps() {
	int i, ii, iii, k, count, kk, n, repeat, endRepeat, 
	    randomStepNumber, randomStepNumber0, randomStep;
	boolean finished, nextUnitIsNotAnEndUnit,
	    createAStandAloneBatchOrder, createASequentialBatchOrder,
	    createAnOrOrder, createAComputationalOrder;

	createAStandAloneBatchOrder=false;
	createASequentialBatchOrder=false;
	createAnOrOrder=false;
	createAComputationalOrder=false;
	orderCount++;
	count=0;

	if(unitList.getCount()==0){
	    System.out.println("totalUnitNumber cannot be 0.");
	    MyExit.exit(1);}

	if(endUnitList.getCount()>0)
	    if(0.05>Globals.env.uniformDblRand.getDoubleWithMin$withMax(0,1))
		createAStandAloneBatchOrder=true;


	if(endUnitList.getCount()>0 && ! createAStandAloneBatchOrder) 
	    // per se sequential batches don't use
	    // end units; this is a switch avoinding
	    // this kind of order generation in case
	    // of plane use of the program 
	    
	    // anyway, the simple test recipes
	    // produced here in the sequential batch
	    // case are conclueded with an end unit
	    if(0.05>Globals.env.uniformDblRand.getDoubleWithMin$withMax(0,1))
		createASequentialBatchOrder=true;

	if(endUnitList.getCount()>0 && ! (createAStandAloneBatchOrder ||
					  createASequentialBatchOrder)) 
	    // per se 'or order' don't use necessarily	    
	    // end units; this is a switch avoinding
	    // this kind of order generation in case
	    // of plane use of the program 
	    if(0.05>Globals.env.uniformDblRand.getDoubleWithMin$withMax(0,1))
		createAnOrOrder=true;

	if(endUnitList.getCount()>0 && ! (createAStandAloneBatchOrder ||
					  createASequentialBatchOrder ||
	                                  createAnOrOrder) ) 
	    // per se a 'computational order' don't use necessarily	    
	    // end units; this is a switch avoinding
	    // this kind of order generation in case
	    // of plane use of the program 
	    if(0.05>Globals.env.uniformDblRand.getDoubleWithMin$withMax(0,1))
		createAComputationalOrder=true;

	// ------------------------------
	// create a stand alone batch order
	if(createAStandAloneBatchOrder)
	    {
		// the recipe scheme of a stand alone batch is:

		// external recipe (with the time specification in seconds 
		// = ticks or in other time units)
		    
		// 11 s 5 / 10 e 101
		// which means: to produce a batch of 5 items in 10 ticks
		// in unit 11 and putting the result in e 101
		    
		// internally, in orderGenerator, with / reported as -2

		// NB NB in sequential batch "/" is substituted with "\"

		// -2 5 10 11 101

		// into anOrder

		// 11 101
		// with a standAloneBatchSpecificationSet object having 
		// the same # of the order and a position parameter relative 
		// to the recipe sequence (here position = 0)

		count=5;
		if(count>maxStepNumber){
		  System.out.println(
                  "OrderGenerator has to produce a stand alone batch order, "+
		  "so set maxStepNumber >= 5");
		  MyExit.exit(1);
		}
		// code of stand alone batch processes
		orderRecipe2[0]=-2;
		// # of seconds = ticks of the whole process
		// (fixed only to simplify this version of the generator)
		orderRecipe2[1]=5;
		// # of items produced (fixed, see above)
		orderRecipe2[2]=10;
		// producing unit
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(endUnitList.getCount(),
					      dictionaryLength-1);
		orderRecipe2[3]=dictionary[n];
		// receiving end unit
		n = Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(0, endUnitList.getCount()-1);
		orderRecipe2[4]=dictionary[n];

		if(StartVEFrame.verbose){
		    System.out.println("Preparing order " 
				       + orderCount + " as a stand alone " +
				       "batch order, with recipe:");
		    for (n=0;n<5;n++)System.out.print(orderRecipe2[n] + " ");
		    System.out.println(" ");}
	    }


	// ------------------------------
	// create a sequential batch order
	if(createASequentialBatchOrder)
	    {
		// the recipe scheme of a stand alone batch is:

		// external recipe (with the time specification in seconds 
		// = ticks or in other time units)
		
		// to chek zero time production in units, we introduce here
		// also a zero time step

		// 10 s 3 11 s 10 \ 5 12 s 1 13 s 0 e 101
		// which means: to produce using steps 10, 11, 12, 13 and
		// putting the result in e 101 (the final e is not 
		// mandatory) 
		// in step 13 the production requires no time: the code
		// that reports this situation is obtained adding 
		// 1000000000 to the production step

		// in unit 11 we have to wait for 5 order to make the step
		// globally in 10 ticks (NB NB here we use \)
		    
		// internally, in orderGenerator, with \ reported as -3

		// 10 10 10 -3 10 5 11 12 1000000013 101

		// into anOrder

		// 10 10 10 11 12 1000000013 101
		// with a sequentialBatchSpecificationSet object having the 
		// same # of the order and a position parameter relative 
		// to the recipe sequence (here position = 3)

		// the structure of these test recipes is fixed; the step 
		// numbers are chosen randomly

		count=10;
		if(count>maxStepNumber){
		  System.out.println(
                  "OrderGenerator has to produce a sequential batch order, "+
		  "so set maxStepNumber >= 10");
		  MyExit.exit(1);
		}
		// production step
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(endUnitList.getCount(),
					      dictionaryLength-1);
		orderRecipe2[0]=dictionary[n];
		orderRecipe2[1]=dictionary[n];
		orderRecipe2[2]=dictionary[n];

		// code of the sequential batch processes
		orderRecipe2[3]=-3;
		// # of seconds = ticks of the whole process
		// (used here as a constant only to simplify this 
		// version of the generator)
		orderRecipe2[4]=10;
		// # of items produced (as a costant, see above)
		orderRecipe2[5]=5;
		// production step (with the sequential batch)
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(endUnitList.getCount(),
					      dictionaryLength-1);
		orderRecipe2[6]=dictionary[n];
		// production step
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(endUnitList.getCount(),
					      dictionaryLength-1);
		orderRecipe2[7]=dictionary[n];
		// production step with 0 time
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(endUnitList.getCount(),
					      dictionaryLength-1);
		orderRecipe2[8]=dictionary[n]+1000000000;
		// receiving end unit
		n = Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(0, endUnitList.getCount()-1);
		orderRecipe2[9]=dictionary[n];

		if(StartVEFrame.verbose)
		    {
		    System.out.println("Preparing order " 
				       + orderCount + " as a sequential " +
				       "batch order, with recipe:");
		    for (n=0;n<10;n++)System.out.print(orderRecipe2[n] + " ");
		    System.out.println(" ");
		    }
	    }


	// ------------------------------
	// create an 'or order' (i.e an order containing an or operator)
	if(createAnOrOrder)
	    {
		// the recipe scheme of an 'or order' is:

		// external recipe (with the time specification in seconds 
		// = ticks or in other time units; to check the code, in the
		// the first branch of the 'or' we have procurement process
		// with 101 below as an end unit)
		
		// 10 s 3 c 1997 1 2 12 s 0 || 1 11 s 2 p 1 101 10 s 1 9 s 2 
		// || 2 c 1995 1 0 1 s 0 14 s 3 || 0 6 s 2
		// where || 1 and || 2 are two nodes of the 'or' sequence (any 
		// number) and || 0 concludes them

		// the code of the first node, instead of 1 can be also 
		// 1000, 1001, 1002 etc.
		// in case of orCriterion 5 (se Or.java), the matrix designated
		// by orMemoryMatrix (in VEFrameModelSwarm.java) is accessed at
		// position 
		// (0,0) if first node is 1 or 1000
		// (1,0) if first node is 1001
		// (2,0) if first node is 1002
		// ...
		// if orCriterion != 5, all the codes above are used as 1

		// in time sequence the recipe is (with 1001 substituting 1)

		// 10 s 3 c 1997 1 2 12 s 0 
                // then 
                //        || 1 11 s 2 p 1 101 10 s 1 9 s 2 || 0 6 s 2
		//        || 2 c 1995 1 0 1 s 0 14 s 3 . . . . . . . . . . .             
		// with || 1 and || 2 branches mutually exclusive

		// at the intermediate level, in orderGenerator, with 
		// || reported as -10 and 
                // c 1997 (see below for computational steps)
		// and with 1001 in place of 1

		// 10 10 10 -1997 1 2 1000000012
		// -10 1001 11 11 -1 1 1 10 9 9 
		// -10 2 -1995 1 0 1000000001 14 14 14 
		// -10 0 6 6

		// zero time steps) as -1997 
		// the computational step has only effects if orCriterion 5
		// is chosen; the computational step (look at 
		// ComputationalAssembler.java, method c1997) places a 1
		// in proper layer of the matrix identified by orMemoryMatrix
		// in VEFrameModelSwarm.java, at (0,0); that value is then 
		// used in Or.java, to choose the first branch of the 'or'
		// process

		// into anOrder

		// 10 10 10 1000000012   11 11 10 9 9    1000000001 
		// 14 14 14    6 6
		// with an OrSpecificationSet object having the 
		// same # of the order and all the position parameters 
		// relative to the 'or' sequence in the recipe 
		// (here position = 4, 9, 13; first position is 0)
		// with a 'p' specification (at position 6) and two 'c'
		// specification (at position 3 and 9)

		// the structure of these test recipes is fixed; the unit 
		// numbers are chosen randomly; the first branch contains
		// a 'p' process, to check its compatibility with 'or'; the
		// second branch contains a 'c' step, newly to chsck its 
		// compatibility

		// alternating data to cross computational step and 'or'
		// processes: c 1997 uses row 1 (and the 'or' process
		// in the same order looks at row 0); c 1996 uses row 0 (and
		// the 'or' process looks at row 1)
		// the goal is that of verifying the capability or the 
		// program to hold the order until the necessary computational
		// step has been done
		if (use0or1==1) use0or1=0;
		else            use0or1=1;


		count=30;
		if(count>maxStepNumber){
		  System.out.println(
                  "OrderGenerator has to produce an 'or' order, "+
		  "so set maxStepNumber >= 30");
		  MyExit.exit(1);
		}
		// producing unit
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(endUnitList.getCount(),
					      dictionaryLength-1);
		orderRecipe2[0]=dictionary[n];
		orderRecipe2[1]=dictionary[n];
		orderRecipe2[2]=dictionary[n];
		// computational process
		if(use0or1==0)orderRecipe2[3]=-1997;
		else          orderRecipe2[3]=-1996;
		orderRecipe2[4]=1;
		orderRecipe2[5]=2;
		//		n= Globals.env.uniformIntRand.
		//  getIntegerWithMin$withMax(endUnitList.getCount(),
		//		      dictionaryLength-1);
		orderRecipe2[6]=12; //n; kept constant and not random to
		                    //   avoid random sequence modification
		                    //   to compare results
		// codes of the or process
		orderRecipe2[7]=-10;
		if(use0or1==0)orderRecipe2[8]= 1;    // same a 1000
		else          orderRecipe2[8]= 1001; // instead of plain 1
		// producing unit
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(endUnitList.getCount(),
					      dictionaryLength-1);
		orderRecipe2[9]=dictionary[n];
		orderRecipe2[10]=dictionary[n];
		// procurement
		orderRecipe2[11]=-1;
		orderRecipe2[12]=1;
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(0,endUnitList.getCount()-1);
		orderRecipe2[13]=n;
		// producing unit
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(endUnitList.getCount(),
					      dictionaryLength-1);
		orderRecipe2[14]=dictionary[n];
		// producing unit
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(endUnitList.getCount(),
					      dictionaryLength-1);
		orderRecipe2[15]=dictionary[n];
		orderRecipe2[16]=dictionary[n];
		// codes of the or process
		orderRecipe2[17]=-10;
		orderRecipe2[18]= 2;
		// code of a computational step embedded here as a test
		// of the strength of the mechanism
		orderRecipe2[19]=-1995;
		orderRecipe2[20]=1;
		orderRecipe2[21]=0;
		orderRecipe2[22]=1000000001;
		// producing unit
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(endUnitList.getCount(),
					      dictionaryLength-1);
		orderRecipe2[23]=dictionary[n];
		orderRecipe2[24]=dictionary[n];
		orderRecipe2[25]=dictionary[n];
		// codes of the or process
		orderRecipe2[26]=-10;
		orderRecipe2[27]= 0;
		// producing unit
		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(endUnitList.getCount(),
					      dictionaryLength-1);
		orderRecipe2[28]=dictionary[n];
		orderRecipe2[29]=dictionary[n];

		if(StartVEFrame.verbose)
		    {
			System.out.println("Preparing order " 
					   + orderCount + " as an Or " +
					   "order, with recipe:");
			for (n=0;n<count;n++)System.out.print(
					     orderRecipe2[n] + " ");
			System.out.println(" ");
		    }
	    }

	// ------------------------------
	// create a bunch of 'computational order' 
	// (i.e orders containing an or operator)
	if(createAComputationalOrder)
	    {
		// the recipe scheme of a 'computational order' is:

		// external recipe (with the time specification in seconds 
		// = ticks or in other time units; to check the code, we are
		// here generating the four following interacting orders
		// with fixed step numbers

		// the code can be checked with or without layers

		// the code containing the code 1999 cannot proceed beyond step
		// 2 if all the other three order have not executed their
		// steps 5 of 6 or 7 with code 1998

		// external recipes of the four cases, with fixed units
		
		// 1 s 1 c 1999 3 0 1 3 2 s 2 3 s 2
		// 1 s 1 c 1998 1 0 5 s 2 
		// 1 s 1 c 1998 1 1 6 s 2 
		// 1 s 1 c 1998 1 3 7 s 2 
		// 
		// where c 1999 or c 1998 are the computational steps linked
		// to unit 2 or 5-6-7 respectively
		// each 'c code' pair is followed by 'n' indicating
		// the number of memory matrixes used ('n' cannot be 0) and
		// by the list of the code of the used memory matrixes

		// internally, in orderGenerator, with 'c code' pairs
		// reported as -code

		// NB pay attention to the position of the computational
		// step, at the end of the time required, followed by a 
		// 0 time step (step number + 1000000000)

		// 1 2 2 -1999 3 0 1 4 1000000002 3 3
		// 1 5 5 -1998 1 0 1000000005 
		// 1 6 6 -1998 1 1 1000000006 
		// 1 7 7 -1998 1 3 1000000007 

		// into anOrder

		// 1  2 2 1000000002  3 3
		// 1  5 5 1000000005
		// 1  6 6 1000000006
		// 1  7 7 1000000007

		// with an ComputationalSpecificationSet object having the 
		// same # of the order, the 'code' of the computation 
		// and the addresses of the used memory matrixes

		// the structure of these test recipes is fixed

		n= Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(1,4);

		if(n==1){
		    // 1 2 2 -1999 3 0 1 4 1000000002 3 3
		    count=11;
		    if(count>maxStepNumber){
			System.out.println(
                  "OrderGenerator has to produce a 'computational' order, "+
		  "so set maxStepNumber >= 11");
			MyExit.exit(1);
		}

		    // producing unit
		    orderRecipe2[0]=1;
		    orderRecipe2[1]=2;
		    orderRecipe2[2]=2;
		    // computational information set
		    orderRecipe2[3]=-1999;
		    orderRecipe2[4]=3;
		    orderRecipe2[5]=0;
		    orderRecipe2[6]=1;
		    orderRecipe2[7]=3;
		    // producing unit
		    orderRecipe2[8]=1000000002;
		    orderRecipe2[9]=3;
		    orderRecipe2[10]=3;

		    if(StartVEFrame.verbose)
			{
			    System.out.println("Preparing order " 
					       + orderCount + 
					       " as a computational"
					       + " order, with recipe:");
			    for (n=0;n<count;n++)System.out.print(
						   orderRecipe2[n] + " ");
			    System.out.println(" ");
			}
		}

		if(n==2){
		    // 1 5 5 -1998 1 0 1000000005 
		    count=7;
		    // producing unit
		    orderRecipe2[0]=1;
		    orderRecipe2[1]=5;
		    orderRecipe2[2]=5;
		    // computational information set
		    orderRecipe2[3]=-1998;
		    orderRecipe2[4]=1;
		    orderRecipe2[5]=0;
		    // producing unit
		    orderRecipe2[6]=1000000005;

		    if(StartVEFrame.verbose)
			{
			    System.out.println("Preparing order " 
					       + orderCount + 
					       " as a computational"
					       + " order, with recipe:");
			    for (n=0;n<count;n++)System.out.print(
						       orderRecipe2[n] + " ");
			    System.out.println(" ");
			}
		}

		if(n==3){
		    // 1 6 6 -1998 1 1 1000000006 
		    count=7;
		    // producing unit
		    orderRecipe2[0]=1;
		    orderRecipe2[1]=6;
		    orderRecipe2[2]=6;
		    // computational information set
		    orderRecipe2[3]=-1998;
		    orderRecipe2[4]=1;
		    orderRecipe2[5]=1;
		    // producing unit
		    orderRecipe2[6]=1000000006;

		    if(StartVEFrame.verbose)
			{
			    System.out.println("Preparing order " 
					       + orderCount + 
					       " as a computational"
					       + " order, with recipe:");
			    for (n=0;n<count;n++)System.out.print(
						    orderRecipe2[n] + " ");
			    System.out.println(" ");
			}
		}

		if(n==4){
		    // 1 7 7 -1998 1 3 1000000007 
		    count=7;
		    // producing unit
		    orderRecipe2[0]=1;
		    orderRecipe2[1]=7;
		    orderRecipe2[2]=7;
		    // computational information set
		    orderRecipe2[3]=-1998;
		    orderRecipe2[4]=1;
		    orderRecipe2[5]=3;
		    // producing unit
		    orderRecipe2[6]=1000000007;

		    if(StartVEFrame.verbose)
			{
			    System.out.println("Preparing order " 
					       + orderCount + 
					       " as a computational"
					       + " order, with recipe:");
			    for (n=0;n<count;n++)System.out.print(
						  orderRecipe2[n] + " ");
			    System.out.println(" ");
			}
		}


	    } // if(createAComputationalOrder)

	// ------------------------------
	// normal form creation process
	if(! (createAStandAloneBatchOrder || createASequentialBatchOrder || 
	      createAnOrOrder || createAComputationalOrder))
	{
	randomStepNumber=Globals.env.uniformIntRand.getIntegerWithMin$withMax
		    (1, maxStepNumber);

	// generating the sequence in the recipe

	// putting 0 in the vector may be useful for future software 
	// modifications
	for (i=0;i<maxStepNumber;i++)orderRecipe1[i]=0;
	for (i=0;i<maxStepNumber;i++)stepLengthInOrder[i]=0;

	finished=false;
	randomStepNumber0=0;

	for (i=0;i<randomStepNumber && ! finished;i++)
	    {
		randomStepNumber0=i+1;

		// generating a random step and stating the probability
		// of concluding the recipe with an "e" end code
		if(0.10 >= Globals.env.uniformDblRand.
		   getDoubleWithMin$withMax(0,1) && 
		   endUnitList.getCount()>0)		
		    randomStep = Globals.env.uniformIntRand.
			getIntegerWithMin$withMax(0, endUnitList.getCount()-1);
		else
		    randomStep = Globals.env.uniformIntRand.
			getIntegerWithMin$withMax(endUnitList.getCount(),
						  dictionaryLength-1);
		orderRecipe1[i]= dictionary[randomStep];

		// check if an end code has been generated
		if (randomStep < endUnitList.getCount())finished=true;

		// generating the length of each step
		if (maxStepLength>1 && ! finished) // end step has lenght 1
		    stepLengthInOrder[i]=
			Globals.env.uniformIntRand.getIntegerWithMin$withMax
			(1, maxStepLength);
		else 
		    stepLengthInOrder[i]=1;
	    }

	// putting together
	// (i) sequences "p" or "c" or ... 
	// (ii) and the lengths of the steps
	count=0;
	for (i=0;i<randomStepNumber0;i++)
	    for (ii=0;ii<stepLengthInOrder[i];ii++)
		{

		    // introducing "p" or "c" sequences

		    // the "p" strucuture is
		    // p k a1 a2 .... ak
		    // where k is the # of items to be procured and 
		    // a1, a2, ..., ak are the codes of those items 
		    // (corresponding to "e" or endUnit codes in sub-recipes
		    // related to internal or external sub-processes

		    // the recipe scheme is
		    // external recipe (with the time specifications and 
		    // a tick corresponding to 1 sec. or to an analogous
		    // minimum fraction of the time
		    
		    // 11 s 3 p 3 101 102 103 22 s 2
		    
		    // internally, in orderGenerator, with p reported as -1

		    // 11 11 11 -1 3 101 102 103 22 22

		    // into anOrder

		    // 11 11 11 22 22
		    // with a procurementSpecifications object having the same 
		    // # of the order and a position parameter relative to the
		    // recipe sequence (here position = 3, starting from 0)

		    
		    if(endUnitList.getCount() > 0)
			{
			    nextUnitIsNotAnEndUnit=true;
			    for(kk=0;kk<endUnitList.getCount();kk++)
				if(orderRecipe1[i]==((EndUnit) endUnitList.
						 atOffset(kk)).getUnitNumber())
				    nextUnitIsNotAnEndUnit=false;
			    
			    // conditions: first item of a step (ii==0)
			    //             next unit number != endUnit number
			    //             (nextUnitIsNotAnEndUnit)
			    if (ii==0 && nextUnitIsNotAnEndUnit &&
				0.10 >= Globals.env.uniformDblRand.
				getDoubleWithMin$withMax(0,1)){

				// length of the "p" or "c" step
				k=Globals.env.uniformIntRand.
				    getIntegerWithMin$withMax
				    (1, endUnitList.getCount());

				// increase the length of orderRecipe2 vector
				orderRecipeTmp = new int[orderRecipe2.
							length+2+k];
				System.arraycopy(orderRecipe2,0,
						 orderRecipeTmp,0,
						 orderRecipe2.length);
				orderRecipe2=orderRecipeTmp;

				//the code of process p
				orderRecipe2[count++]= -1; 

				// the length
				orderRecipe2[count++]=k;   
				
				// k codes, chosen randomly
				for (iii=0;iii<k;iii++)   
				    orderRecipe2[count++]=
					((EndUnit) endUnitList.
					 atOffset(
						  Globals.env.uniformIntRand.
						  getIntegerWithMin$withMax
						  (0, endUnitList.getCount()-1)
						  )).getUnitNumber();
			    }
			}

		    // copying normal steps
		    orderRecipe2[count++]=orderRecipe1[i];

		}
	
	if(StartVEFrame.verbose)
	{
	    for (iii=0; iii<count;iii++)
		System.out.print(orderRecipe2[iii]+ " ");
	    System.out.println(" internal recipe for order " + orderCount);
	}

	// ------------------------------
	// concluding normal order form generation
        }

	// choosing the layer of the order(s) generated 
	// (from 0 to totalLayerNumber-1)
	if(totalLayerNumber>1)
	    if(0.03>Globals.env.uniformDblRand.
	       getDoubleWithMin$withMax(0,1))
		layerNumber=Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(0, totalLayerNumber-1);

/*      // to be activate to generate comparative test with one or more 
        // layers and insensitive units
	// this second layer generator has no effect, but in case of only one
	// layer it reproduces the same sequence of random numbers
	if(totalLayerNumber==1)
	    if(0.03>Globals.env.uniformDblRand.
	       getDoubleWithMin$withMax(0,1))
		Globals.env.uniformIntRand.
		    getIntegerWithMin$withMax(0, 1); // don't change this
	                                             // 0-1 interval
*/

	// repeating the order generation (in sequential batch order case)
	endRepeat=1;

	// deliberately the repetitions here are five, being also the
	// the whole batch composed of five order; due to the delay in 
	// assigning order to the unit using the sequential batch process,
	// the sequential batch assembler cannot anyway clear the batch 
	// for production at the first check (this delay in clearing is 
	// useful for testing purposes)
	if(createASequentialBatchOrder)endRepeat=5;

	for(repeat=0;repeat<endRepeat;repeat++){

	// creating an order
        // vector dimension in anOrder is count: count is a step ahead (see 
        // last count++ operation) but starting with 0 is the correct number
        // of elements
	anOrder = new Order(getZone(), orderCount,
			    Globals.env.getCurrentTime(),
			    count, orderRecipe2, 
			    vEFrameModelSwarm, endUnitList);

	// setting the layer (from 0 to totalLayerNumber-1)
	anOrder.setOrderLayer(layerNumber);
	if(createAComputationalOrder)anOrder.setRecipeName("comp");

	if(StartVEFrame.verbose)
	{
	    for (iii=0; iii<anOrder.orderRecipe.length;iii++)
		System.out.print(anOrder.orderRecipe[iii]+ " ");
	    System.out.println(" internal recipe into the order " +
			       orderCount);
	}


        // add the active orders to the general order list (they will be
	// eliminated when dropped in a unit, being finished); this
	// list has been introduced for accounting purposes [may be it would
	// be better subtitute it with a get to the units to know their
	// waiting lists] 
        orderList.addLast(anOrder);

	/**
	 * sending the order to the first production unit
	 * (we are acting as the Front End of the VE)
	 */
	assigningTool.assign(anOrder);
/*	unitNotFound=true;
	for (i=0;i<unitList.getCount() && unitNotFound;i++)
	    {
		aUnit=(Unit) unitList.atOffset(i);
		for (iii=0;iii<aUnit.getNOfPhasesToDealWith() &&  
			 unitNotFound;iii++)
		    {
		       if(anOrder.getNextStep()==aUnit.getProductionPhase(iii))
			    {
				unitNotFound=false;
				aUnit.setWaitingList(anOrder);
			    }
		    }
	    }

	// considering also endUnits
	for (i=0;i<endUnitList.getCount() && unitNotFound;i++)
	    {
		anEndUnit=(EndUnit) endUnitList.atOffset(i);
		if(anOrder.getNextStep()==anEndUnit.getUnitNumber())
		    {
			unitNotFound=false;
			anEndUnit.setTemporaryList(anOrder);
		    }
	    }



	if(unitNotFound){
	    System.out.println("OrderGenerator internal error: receiving "+
			       "unit does not exixts");
	    MyExit.exit(1);
	}
*/
	// finish repeating
	if(endRepeat>1 && repeat<4)orderCount++;
	}

    }

}
