// PTHistogramPlottable.java


/** The interface to be implemented in classes referred from PTHistogram 
 *  class<br><br>
 *
 *  PTHistogram can deal with up to three list of objects from which to obtain
 *  the data to be plotted; from the objects contained in the first or unique 
 *  list (second or third lists are optional) we have also to obtain the 
 *  captions for the various bar in the X axis<br><br>
 *
 *  an example of getValueToPlot() method in an object extending this interface
 *  is<br><br>
 *     public double getValueToPlot()<br>
 *         {<br>
 *                  return (double) XXXX;<br>
 *                  // where XXXX is a value or a method getting a value<br>
 *         }
 *<br><br>
 *  an example of getLabel() method in an object extending this interface,
 *  if the object is included in the fist or unique list sent to
 *  PTHistogram, is<br><br>
 *     public String getLabel()<br>
 *         {<br>
 *                   return "" + YYYY;<br>
 *                   // where YYYY is a numerical value or a method getting<br>
 *                   // a numerical value (the trick "" +  is used<br>
 *                   // to convert a numerical field into a string; if we<br>
 *                   // have directly a string, the trick is unnecessary)<br>
 *         }<br><br>
 *
 *  if the object is included in the second or third list, simply use<br><br>
 *                   return "";<br>
 *  in getLabel() body<br><br> 
 *
 *  @author From a Michele Sonnessa's idea, with comments of Pietro Terna
 *
 */
public interface PTHistogramPlottable {

    public String getLabel();
    public double getValueToPlot();
}
