// SequentialBatchSpecificationSet.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;


/**
 * The container generated by an order and containing the spefications about
 * a sequential batch process
 *
 * @author Pietro Terna
 */
public class SequentialBatchSpecificationSet  extends SwarmObjectImpl{

    /** number (i.e. that of the order which has generated this object */
    public int myOrderNumber;
    /** the position of this sequence in the recipe */
    public int pos;
    /** the specifications */
    public int productionTimeInTicks;
    public int quantityInSequentialBatch;

  /**
   * constructor for SequentialBatchSpecificationSet
   */
  public SequentialBatchSpecificationSet (Zone aZone, int n, int p) {

     // Call the constructor for the parent class
     super(aZone);

     myOrderNumber=n;
     pos=p;

	if(StartVEFrame.verbose)
	    System.out.println("Sequential b. spec. created " +
			       "(by order " + myOrderNumber + " at pos. " + 
			       pos + "; NB first pos. is 0)");
  }

    /** setting the sequential batch specifications */
    public void setSpecifications(int k1, int k2)
    {
	productionTimeInTicks = k1;
	quantityInSequentialBatch = k2;

	if(StartVEFrame.verbose)
	    {
	      System.out.println(
               "Sequential batch production time in ticks " +
	       productionTimeInTicks);
	      System.out.println(
               "Quantity in sequential batch production " +
	       quantityInSequentialBatch + " orderNumber "+myOrderNumber);
	    }
    }

    /** ???????? decrease production time */
    public int decreaseProductionTime(){

	if(StartVEFrame.verbose)
	    System.out.println(
             "Residual production time in stand alone production set from"+
	     " order " + myOrderNumber+" at pos " + pos +": " + 
	     productionTimeInTicks);
	productionTimeInTicks--;
	return productionTimeInTicks;
    }


    /**
     * returning the number of the order of this procurement specification
     */
    public int getNumber () {
	return myOrderNumber;
   }

    /**
     * returning the position of this procurement specification into its recipe
     */
    public int getPositionInRecipe() {
	return pos;
   }

    /** returning production time in ticks (original or residual) */
    public int getProductionTimeInTicks(){
	return productionTimeInTicks;
    }

    /** settinging production time in ticks (residual) */
    public void setProductionTimeInTicks(int t){
	productionTimeInTicks=t;
    }

    /** returning quantity in sequential batch */
    public int getQuantityInSequentialBatch(){
	return quantityInSequentialBatch;
    }

}
